GMAP/GSNAP 2011-03-28.v3 for Windows
====================================

Source Code Modifications
-------------------------

The GMAP/GSNAP package was designed for compilation and use under a
UNIX-like operating system.  Although autoconf eliminates many
portability problems, and MinGW-w64 provides some of the necessary
UNIX facilities, some changes to the source code were nonetheless required
to compile GMAP/GSNAP for Windows:

1.	Calls to the deprecated functions `index()` and `rindex()` were replaced
by calls to `strchr()` and `strrchr()`, respectively, in the following files:
	
		src/datadir.c
		src/genome-write.c
		src/get-genome.c
		src/gmap.c
		src/gmapindex.c
		src/iit_get.c
		src/iit_store.c
		src/parserange.c
		src/sequence.c
		src/shortread.c

2.	Autoconf was already being used to detect support for `mmap()`, but
several files did not check for `mmap` support before including
`<sys/mman.h>`.  `#ifdef HAVE_MMAP` was inserted where necessary to 
include this header only if `mmap` is supported on the target system.  The
affected files were:

		src/access.c
		src/cmetindex.c
		src/genome.c
		src/iit-read.c
		src/indexdb.c
		src/indexdb_dibase.c
		src/snpindex.c

3.	In `src/cmetindex.c` and `src/snpindex.c`, `Access_mmap()` was
replaced with the similar `Access_allocated()` when `HAVE_MMAP` is
undefined.
		
4.	In `src/indexdbdef.h`, `#ifdef HAVE_PTHREAD` was added to
include `<pthread.h>` only if POSIX threads are supported on the
target system.

5.	In `src/datadir.c`, the inclusion of `<pwd.h>` was removed
on MinGW, where that header does not exist.  The implementation
of `find_homedir_config()` was replaced (on MinGW only) with a
version that uses Windows environment variables rather than
the `getpwnam()` function to determine the user's home directory.

6.	In `src/stopwatch.c` (on MinGW only), the implementation of the
stopwatch object was modified to use system primitives available
on MinGW, including the `clock()` function, defined in `<time.h>`,
rather than the `times()` function, defined in `<sys/times.h>` on
UNIX-like systems.  All four stopwatch functions (`Stopwatch_new()`,
`Stopwatch_free()`, `Stopwatch_start()`, and `Stopwatch_stop()`) were
implemented separately for MinGW.

7.	In `main()` in `src/gmapindex.c`, `stdin` was set to binary mode
wherever compressed data must be read from `stdin`, namely, when

		action == UNCOMPRESS ||
		action == OFFSETS ||
		action == POSITIONS


Additions
---------

Some additional code was added to replace functionality that
comes standard on most UNIX-like systems:

1.	An implementation of `mmap()`, `munmap()`, `msync()`, and `getpagesize()` was
added, comprising the files `windows/sys/mman.h` and `windows/sys/mman.c`.
This implementation uses Windows' built-in memory mapping functions
(`CreateFileMapping()` and `MapViewOfFileEx()`, among others).  The autotools
build system was updated to use this implementation on Windows.

	On Windows, one must create a "file mapping object" from a file
before mapping a "view" of that object into memory.
A process accesses a mapping view via a pointer, but a file mapping
object is only accessible via its "handle."  Windows does not make visible
the associations between file mapping handles and address ranges.  However,
in order to properly unmap a file, its mapping handle must be closed.  As
the POSIX-style `munmap()` only takes as arguments a pointer to a mapping
and its length, the Windows `munmap()` implementation must keep track
of the handle corresponding to a mapping from the time it is created using
`mmap()` until it must be closed using `munmap()`.  In this implemenation,
that functionality is provided by a simple hash table with pointers as keys
and values that hold mapping information.
