static char rcsid[] = "$Id: stopwatch.c 27450 2010-08-05 19:02:48Z twu $";
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "stopwatch.h"
#include "mem.h"

#if defined(__MINGW64__) || defined(__MINGW32__)
#include <time.h>

#define T Stopwatch_T
struct T
{
	clock_t start;
	clock_t stop;
};

T
Stopwatch_new() {
	T new_watch = (T) MALLOC(sizeof(*new_watch));
  return new_watch;
}

void
Stopwatch_free(T *watch) {
	if (*watch)
		FREE(*watch);
	return;
}

void
Stopwatch_start(T watch) {
	if (watch != NULL)
		watch->start = clock();
}

double 
Stopwatch_stop(T watch) {
	if (watch != NULL)
	{
		watch->stop = clock();
		return ((double) watch->stop - watch->start) / ((double) CLOCKS_PER_SEC);
	}
}

#else // *NIX
#ifdef HAVE_UNISTD_H
#include <unistd.h>		/* For sysconf */
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>		/* For sys/times.h under AT&T System V Interface */
#endif
#include <sys/times.h>

#define T Stopwatch_T
struct T {
  struct tms start;
  struct tms stop;
  clock_t start_elapsed;
  clock_t stop_elapsed;
};

T
Stopwatch_new () {
  T new = (T) MALLOC(sizeof(*new));
  return new;
}

void
Stopwatch_free (T *old) {
  if (*old) {
    FREE(*old);
  }
  return;
}

void
Stopwatch_start (T this) {
  if (this != NULL) {
    this->start_elapsed = times(&this->start);
  }
  return;
}

/* Returns user seconds elapsed since start of process. */
/* struct tms is defined in <sys/times.h> (see man times); tms values are
   in "clock ticks per second", CLK_TCK */
double 
Stopwatch_stop (T this) {
  long clk_tck = sysconf(_SC_CLK_TCK);

  if (this == NULL) {
    return 0.0;
  } else {
    this->stop_elapsed = times(&this->stop);

    /* user time is in stop.tms_utime */
    return (double) (this->stop_elapsed - this->start_elapsed)/(double) clk_tck;
  }
}
#endif

