#include <boost/test/unit_test.hpp>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <string>
using namespace std;

// prototypes of functions to test:
extern "C" void *Access_allocated(size_t *len, double *seconds, char *filename, size_t eltsize);
extern "C" void *Access_mmap(int *fd, size_t *len, char *filename, size_t eltsize, bool randomp);

struct TestTemp
{
	int fd;
	char tmpname[L_tmpnam];
	char *contents = "Do not look directly at this temporary file.\000";
	size_t len = 45;
	TestTemp()
	{
		tmpnam(tmpname);
		fd = open(tmpname, O_RDWR | O_CREAT);
		write(fd, contents, len);
		close(fd);
		fd = open(tmpname, O_RDWR);
	}
	~TestTemp()
	{
		close(fd);
		remove(tmpname);
	}
};

BOOST_FIXTURE_TEST_SUITE(AccessAllocated, TestTemp)

BOOST_AUTO_TEST_CASE(accessAllocatedReturnsValidPointer)
{
	size_t inferred_len;
	double seconds;
	void *mem = Access_allocated(&inferred_len, &seconds, tmpname, sizeof(char));
	BOOST_REQUIRE_NE(mem, NULL);
}

BOOST_AUTO_TEST_CASE(accessAllocatedInfersProperFileSize)
{
	size_t inferred_len;
	double seconds;
	void *mem = Access_allocated(&inferred_len, &seconds, tmpname, sizeof(char));
	BOOST_REQUIRE_EQUAL(len, inferred_len);
}

BOOST_AUTO_TEST_CASE(accessAllocatedBehavesLikeAccessMMap)
{
	size_t len_allocated;
	size_t len_mmapped;
	double seconds;
	int fd;
	char *alloc_str = (char *)Access_allocated(&len_allocated, &seconds, tmpname, sizeof(char));
	char *mmap_str = (char *)Access_mmap (&fd, &len_mmapped, tmpname, sizeof(char), false);
	BOOST_REQUIRE_EQUAL(len_allocated, len_mmapped);
	BOOST_REQUIRE_EQUAL((string)alloc_str, (string)mmap_str);
}

BOOST_AUTO_TEST_SUITE_END()