#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include <unistd.h>

// prototypes of functions to test:
struct Stopwatch_T;
extern "C" struct Stopwatch_T *Stopwatch_new();
extern "C" void	Stopwatch_free(struct Stopwatch_T **old);
extern "C" void	Stopwatch_start(struct Stopwatch_T *watch);
extern "C" double Stopwatch_stop(struct Stopwatch_T *watch);


BOOST_AUTO_TEST_SUITE(Stopwatch)

BOOST_AUTO_TEST_CASE(stopwatchCreateAndDestroy)
{
	struct Stopwatch_T *watch = Stopwatch_new();
	BOOST_REQUIRE(watch != NULL);
	Stopwatch_free(&watch);
}

BOOST_AUTO_TEST_CASE(stopwatchTimeIncreases)
{
	struct Stopwatch_T *watch1 = Stopwatch_new();
	struct Stopwatch_T *watch2 = Stopwatch_new();
	Stopwatch_start(watch1);
	Stopwatch_start(watch2);
	double less = Stopwatch_stop(watch2);
	double more = Stopwatch_stop(watch1);
	BOOST_REQUIRE_LE(less, more);
	Stopwatch_free(&watch1);
	Stopwatch_free(&watch2);
}

BOOST_AUTO_TEST_CASE(stopwatchTimeIsReasonable)
{
	struct Stopwatch_T *watch = Stopwatch_new();
	Stopwatch_start(watch);
	usleep(999999);
	double seconds = Stopwatch_stop(watch);
	// check that time measured is within 0.1% (i.e., 1ms) of 0.999999 secs.
	BOOST_REQUIRE_CLOSE(seconds, 0.999999, 0.1);
	Stopwatch_free(&watch);
}

BOOST_AUTO_TEST_SUITE_END()