MaqView for Windows
===================

Source Code Modifications
-------------------------

MaqView was designed for compilation under UNIX/POSIX-compliant systems.
To compile it under Windows, some changes to its source code were made:

1.	In `btree.h` and `zrio.h`, `#include <stdint.h>` was added to provide declarations of
	fixed-width integer types.
2.	In `maqview.h`, `stdhashc.cc`, and `stdhashc.h`, the type `u_int32_t` was replaced with
	`uint32_t` to be more compatible with Standard C.
3.	In `stdhash.hh`, line 496 was changed from
		
		int ret = direct_insert_aux(key, this->n_capacity, this->keys, this->flags, &i);
	
	to
		
		int ret = this->direct_insert_aux(key, this->n_capacity, this->keys, this->flags, &i);
	
	to resolve an ambiguity in the templated subclass `hash_map_misc`.

4.	In `maqview.h` and `view_panel.c`, the structure type `SelectObject`
	was changed to `MVSelectObject` so as not to conflict with a `SelectObject()` function
	included from a Windows header file.

5.	Various files were set to open in binary mode.  The following lines were affected:

		cns_cache.c: 146
		maqmap_index.c: 167, 223, 281
		maqmap_index_main.c: 184
		zrio.c: 19, 52, 566, 575
	
	For files opened with the POSIX `open()` function, rather than `fopen()`, the binary
	mode flag `O_BINARY` is only available on MinGW (and must be included from `<fcntl.h>`.
	In `zrio.c`, this flag was defined to zero by default with
	
		#ifndef O_BINARY
		#define O_BINARY 0
		#endif
	
	so that files can be opened for reading in binary mode with
	
		`open(filename, O_RDONLY | O_BINARY)`
		
	In `zrio_main.c`, `stdout` was set to binary mode on MinGW with
	
		#if defined(__MINGW64__) || defined(__MINGW32__)
			// Under MinGW/Windows, opening stdout in text mode causes problems...
			setmode(fileno(stdout), O_BINARY);
		#endif
	
	in `main()` (on lines 53-56).  `<fcntl.h>` was included here as well.
	
6.	`Makefile.generic` was modified to define `MAQ_LONGREADS`.  This
	fixes a bug which causes a segmentation fault while `maqindex` reads
	a compressed alignment file.
7.	In `view_panel.c`, in function `closeViewPanel()`, the lines which execute
	`glutDestroyWindow()` are set not to compile on MinGW.  This fixes a bug in which
	MaqView crashes when the user tries to close it using the "close window" button on Windows.
8.	In `socket_view.h`, `<winsock.h>` is included rather than `<sys/socket.h>`, `<netinet/in.h>`,
	`<arpa/inet.h>`, and `<netdb.h>` on Windows.
9.	The autotools build system was modified to work on Windows.  `config.sub`, `depcomp`, and `missing`
	were changed from Unix symbolic links to copies of the corresponding target shell scripts from
	Cygwin.  In `Makefile.am`, `$(SOCKLIBS)` is added to `maqindex_socks_LDADD`.
	Finally, `configure.ac` is set to check for `${host_os} = mingw` and to set corresponding `CFLAGS`,
	`CPPFLAGS`, `CXXFLAGS`, `GLLIBS`, and `SOCKLIBS`.
	
Build Settings
--------------

In both `configure.ac` and `Makefile.generic`, the following should be set for MinGW compilation:

*	`GLLIBS` should be set to

		-Wl,-Bstatic -lglut -lstdc++ -Wl,-Bdynamic -lopengl32 -lglu32 -lgdi32 -lwinmm -Wl,--subsystem,windows

*	`-static -DFREEGLUT_STATIC` should be added to `CFLAGS`.

*	`CXXFLAGS` should be set to `CFLAGS`.

In `configure.ac`, `SOCKLIBS` should be set to `-lws2_32`.

